package com.dzeio.charts

import android.content.Context
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.Paint
import android.graphics.RectF
import android.util.AttributeSet
import android.util.Log
import android.view.MotionEvent
import android.view.View
import com.dzeio.charts.axis.XAxis
import com.dzeio.charts.axis.YAxis
import com.dzeio.charts.components.ChartScroll
import com.dzeio.charts.series.SerieAbstract
import kotlin.math.max
import kotlin.math.min

class ChartView @JvmOverloads constructor(context: Context?, attrs: AttributeSet? = null) :
    View(context, attrs) {

    companion object {
        const val TAG = "DzeioCharts/ChartView"
    }

    var debug = false

    val xAxis = XAxis<Float>()

    val yAxis = YAxis<Float>(this)

    val animation = Animation()

    val scroller = ChartScroll(this).apply {
        setOnChartMoved { movementX, movementY ->
//            Log.d(TAG, "scrolled: $movementX")
            movementOffset = movementX / 100
            refresh()
        }
        setOnZoomChanged {
            Log.d(TAG, "New Zoom: $it")
            zoom = (it * 1.2).toFloat()
            refresh()
        }
    }

    private val animator: Runnable = object : Runnable {
        override fun run() {
            var needNewFrame = false
            for (serie in series) {
                val result = serie.onUpdate()
                if (result) {
                    needNewFrame = true
                }
            }
            if (needNewFrame) {
                postDelayed(this, animation.getDelay().toLong())
            }
            invalidate()
        }
    }

    /**
     * global padding
     */
    var padding: Float = 8f

    var series: ArrayList<SerieAbstract> = arrayListOf()
        set(value) {
            for (serie in value) {
                serie.view = this
            }
            field = value
        }

    /**
     * Number of entries displayed at the same time
     */
    private var zoom = 100f

    var movementOffset: Float = 0f

    private val rectF = RectF()
    private val otherUseRectF = RectF()

    fun refresh() {
        for (serie in series) {
            serie.prepareData()
        }
        rectF.set(
            padding,
            padding,
            measuredWidth - padding - yAxis.getWidth() - padding,
            height - padding
        )

        removeCallbacks(animator)
        post(animator)
    }

    private val fgPaint: Paint = Paint().also {
        it.isAntiAlias = true
        it.color = Color.parseColor("#123456")
    }

    override fun onDraw(canvas: Canvas) {

        if (yAxis.legendEnabled) {
            yAxis.display(canvas, measuredWidth, height)
        }

        for (serie in series) {
            serie.displayData(canvas, rectF)
        }
//        canvas.drawRect(
//            measuredWidth - padding - yAxis.getWidth(),
//            0f,
//            measuredWidth - padding,
//            height - padding,
//            fgPaint
//        )
        super.onDraw(canvas)
    }

    override fun onTouchEvent(event: MotionEvent): Boolean {
        performClick()
        return scroller.onTouchEvent(event)
    }

    fun getXOffset(): Int {
//        Log.d(
//            TAG,
//            "baseOffset: ${xAxis.baseOffset}, mOffset: $movementOffset = ${xAxis.baseOffset + movementOffset}"
//        )
//        Log.d(
//            TAG,
//            "longestOffset: ${longestSerie()}, displayedEntries: ${getDisplayedEntries()} = ${longestSerie() - getDisplayedEntries()}"
//        )
        return min(
            max(0f, xAxis.baseOffset + movementOffset).toInt(),
            max(0, getCalculatedMax() - getDisplayedEntries())
        )
    }

    fun getDisplayedEntries(): Int {
//        Log.d(TAG, "Number of entries displayed ${list.size}, ${xAxis.entriesDisplayed} + (($zoom - 100) * 10) = ${xAxis.entriesDisplayed + ((zoom - 100) * 10).toInt()}")
        return max(0, xAxis.entriesDisplayed + ((zoom - 100) * 10).toInt())
    }

    fun getCalculatedMax(): Int {
        var size = 0
        for (serie in series) {
            if (serie.datas.size > size) size = serie.datas.size
        }
        return size
    }

    fun getXMax(displayedOnly: Boolean = false): Float {
        var max = 0f
        for (serie in series) {
            val res = serie.getYMax(displayedOnly)
            if (max < res) max = res
        }
        return max
    }
}
